-----------------------------------------------------------------------------------------------
-- Client Lua Script for Raid_Sheet
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- Raid_Sheet Module Definition
-----------------------------------------------------------------------------------------------
local Raid_Sheet = {}
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

local RaidList = {
	"Datascape",
	"RMT",
	}
local RMTList = {
	"Swabbie",
	}
 
local DatascapeList = {
	"SD",
	"Limbo",
	"Lattice",
	"Gloomclaw",
	"Earth",
	"Life",
	"Water",
	"Avatus",
	}
local Guilds = {
	"Elysium",
	"Bloodpact",
	}
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Raid_Sheet:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
	
    -- initialize variables here
    return o
end

function Raid_Sheet:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- Raid_Sheet OnLoad
-----------------------------------------------------------------------------------------------
function Raid_Sheet:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("Raid_Sheet.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	Apollo.LoadSprites("Sprites.xml")
end

-----------------------------------------------------------------------------------------------
-- Raid_Sheet OnDocLoaded
-----------------------------------------------------------------------------------------------
function Raid_Sheet:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "Raid_SheetForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		Apollo.LoadSprites("Sprites.xml")
		
	    self.wndMain:Show(false, true)
		self.Raid = nil
		self.Boss = nil
		self.Guild = nil
		self.save = {
			["Datascape"] = {
				["Water"] = {
					["Elysium"] = nil,
					["Bloodpact"] = nil,
					},
				["Life"] = {
					["Elysium"] = nil,
					["Bloodpact"] = nil,
					},
				["Earth"] = {
					["Elysium"] = nil,
					["Bloodpact"] = nil,
					},
				["Gloomclaw"] = {
					["Elysium"] = nil,
					["Bloodpact"] = nil,
					},
				["Lattice"] = {
					["Elysium"] = nil,
					["Bloodpact"] = nil,
					},
				["Limbo"] = {
					["Elysium"] = nil,
					["Bloodpact"] = nil,
					},
				["SD"] = {
					["Elysium"] = nil,
					["Bloodpact"] = nil,
					},
				["Avatus"] = {
					["Elysium"] = nil,
					["Bloodpact"] = nil,
					},
				},
			["RMT"] = {
				["Swabbie"] = {
					["Elysium"] = nil,
					["Bloodpact"] = nil,
					},
				},
			}

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterEventHandler("ChatMessage", "ControlChatMessage", self)
		Apollo.RegisterEventHandler("Group_Remove", "OnUpdateRooster", self)
		Apollo.RegisterEventHandler("Group_Add", "OnUpdateRooster", self)
		Apollo.RegisterEventHandler("Group_MemberFlagsChanged", "OnUpdateRooster", self)
		
		--Commands
		Apollo.RegisterSlashCommand("rs", "OnRaid_SheetOn", self)
		Apollo.RegisterSlashCommand("RaidSheet", "OnRaid_SheetOn", self)


		-- Do additional Addon initialization here
		self:SetRaidList()
	end
end

-----------------------------------------------------------------------------------------------
-- Raid_Sheet Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/rs"
function Raid_Sheet:OnRaid_SheetOn()
	self.wndMain:Invoke() -- show the window
end

function Raid_Sheet:SetRaidList()

	local wndRaidList = self.wndMain:FindChild("RaidList")
	
	for k,v in pairs(RaidList) do
		local Raid = Apollo.LoadForm(self.xmlDoc, "RaidBox", wndRaidList, self)
		Raid:FindChild("Name"):SetText(v)
	end
	
	wndRaidList:ArrangeChildrenVert(0)
end

function Raid_Sheet:ShowBossList(Raid)
	self.Raid = Raid
	local wndBossList = self.wndMain:FindChild("BossList")
	wndBossList:DestroyChildren()
	
	if Raid == "Datascape" then
		for k,v in pairs(DatascapeList) do
			local Boss = Apollo.LoadForm(self.xmlDoc, "BossBox", wndBossList, self)
			Boss:FindChild("Name"):SetText(v)
		end
	elseif Raid == "RMT" then
		for k,v in pairs(RMTList) do
			local Boss = Apollo.LoadForm(self.xmlDoc, "BossBox", wndBossList, self)
			Boss:FindChild("Name"):SetText(v)
		end
	end
	
	wndBossList:ArrangeChildrenVert(0)
end
-----------------------------------------------------------------------------------------------
-- Raid_SheetForm Functions
-----------------------------------------------------------------------------------------------
-- when the Postbutton is clicked
function Raid_Sheet:OnOK()
	local SaveString = nil;
	if self.Raid == "Datascape" then
		if self.Boss == "Gloomclaw" then
			if self.Guild == "Elysium" then
				local wndSheet = self.wndMain:FindChild(self.Boss.."_"..self.Guild)
				if wndSheet ~= nil then
					SaveString = "RS".."_"..self.Raid.."_"..self.Boss.."_"..self.Guild
					local slot = 13
					for i = 1, slot do 
						SaveString = SaveString.."_"
						..wndSheet:FindChild("L"..i):GetText().."_"
						..wndSheet:FindChild("R"..i):GetText()
					end
					SaveString = SaveString.."_"
					ChatSystemLib.Command("/p "..SaveString)
				end
			else
			end
		elseif self.Boss == "Lattice" then
			if self.Guild == "Elysium" then
				local wndSheet = self.wndMain:FindChild(self.Boss.."_"..self.Guild)
				if wndSheet ~= nil then
					SaveString = "RS".."_"..self.Raid.."_"..self.Boss.."_"..self.Guild
					local slot = 7
					for i = 1, slot do 
						SaveString = SaveString.."_"
						..wndSheet:FindChild("N"..i):GetText().."_"
						..wndSheet:FindChild("S"..i):GetText().."_"
						..wndSheet:FindChild("D"..i):GetText()
					end
					SaveString = SaveString.."_"..wndSheet:FindChild("D8"):GetText().."_"
					ChatSystemLib.Command("/p "..SaveString)
				end
			else
			end
		elseif self.Boss == "Limbo" then
			if self.Guild == "Elysium" then
				local wndSheet = self.wndMain:FindChild(self.Boss.."_"..self.Guild)
				if wndSheet ~= nil then
					SaveString = "RS".."_"..self.Raid.."_"..self.Boss.."_"..self.Guild
					local slot = 7
					for i = 1, slot do 
						SaveString = SaveString.."_"
						..wndSheet:FindChild("G1"..i):GetText().."_"
						..wndSheet:FindChild("G2"..i):GetText().."_"
						..wndSheet:FindChild("G3"..i):GetText()
					end
					SaveString = SaveString.."_"
					ChatSystemLib.Command("/p "..SaveString)
				end
			else
			end
		elseif self.Boss == "SD" then
			if self.Guild == "Elysium" then
				local wndSheet = self.wndMain:FindChild(self.Boss.."_"..self.Guild)
				if wndSheet ~= nil then
					SaveString = "RS".."_"..self.Raid.."_"..self.Boss.."_"..self.Guild
					--North/South
					for i = 1, 8 do 
						SaveString = SaveString.."_"
						..wndSheet:FindChild("N"..i):GetText().."_"
						..wndSheet:FindChild("S"..i):GetText()
					end
					--control
					for i = 1, 6 do 
						SaveString = SaveString.."_"
						..wndSheet:FindChild("C"..i):GetText()
					end
					SaveString = SaveString.."_"
					ChatSystemLib.Command("/p "..SaveString)
				end
			else
			end
		elseif self.Boss == "Avatus" then
			if self.Guild == "Elysium" then
				local wndSheet = self.wndMain:FindChild(self.Boss.."_"..self.Guild)
				if wndSheet ~= nil then
					SaveString = "RS".."_"..self.Raid.."_"..self.Boss.."_"..self.Guild
					for i = 1, 10 do 
						SaveString = SaveString.."_"
						..wndSheet:FindChild("Y"..i):GetText().."_"
						..wndSheet:FindChild("G"..i):GetText().."_"
						..wndSheet:FindChild("R"..i):GetText().."_"
						..wndSheet:FindChild("B"..i):GetText()
					end
					SaveString = SaveString.."_"
					ChatSystemLib.Command("/p "..SaveString)
				end
			elseif self.Guild == "Bloodpact" then
				local wndSheet = self.wndMain:FindChild(self.Boss.."_"..self.Guild)
				if wndSheet ~= nil then
					SaveString = "RS".."_"..self.Raid.."_"..self.Boss.."_"..self.Guild
					for i = 1, 10 do 
						SaveString = SaveString.."_"
						..wndSheet:FindChild("Y"..i):GetText().."_"
						..wndSheet:FindChild("G"..i):GetText().."_"
						..wndSheet:FindChild("R"..i):GetText().."_"
						..wndSheet:FindChild("B"..i):GetText()
					end
					SaveString = SaveString.."_"
					..wndSheet:FindChild("YB1"):GetText().."_"
					..wndSheet:FindChild("YB2"):GetText().."_"
					ChatSystemLib.Command("/p "..SaveString)
				end
			else
			end
		else
		end
	else
		if self.Boss == "Swabbie" then
			if self.Guild == "Elysium" then
				local wndSheet = self.wndMain:FindChild(self.Boss.."_"..self.Guild)
				if wndSheet ~= nil then
					SaveString = "RS".."_"..self.Raid.."_"..self.Boss.."_"..self.Guild
					local slot = 7
					for i = 1, slot do 
						SaveString = SaveString.."_"
						..wndSheet:FindChild("G1"..i):GetText().."_"
						..wndSheet:FindChild("G2"..i):GetText().."_"
						..wndSheet:FindChild("G3"..i):GetText()
					end
					SaveString = SaveString.."_"
					ChatSystemLib.Command("/p "..SaveString)
				end
			else
			end
		else
		end
	end	
end

-- when the Cancel button is clicked
function Raid_Sheet:OnCancel()
	self.wndMain:Close() -- hide the window
end

---------------------------------------------------------------------------------------------------

-- RaidBox Functions
---------------------------------------------------------------------------------------------------

function Raid_Sheet:OnClickRaid( wndHandler, wndControl)
	local Raid = wndHandler:GetParent():FindChild("Name"):GetText()
	self:ShowBossList(Raid)
end


function Raid_Sheet:OnClearSheet( wndHandler, wndControl, eMouseButton )
	
	if self.Guild == nil or self.Boss == nil or self.Raid == nil then
		return
	end
	
	local wndSheet = self.wndMain:FindChild("RaidWindow")
	wndSheet:FindChild("RaidWindow"):DestroyChildren()
	self.save[self.Raid][self.Boss][self.Guild] = nil
	Apollo.LoadForm(self.xmlDoc, self.Boss.."_"..self.Guild , wndSheet, self)
end


---------------------------------------------------------------------------------------------------
-- GuildBox Functions
---------------------------------------------------------------------------------------------------

function Raid_Sheet:OnClickGuild( wndHandler, wndControl, eMouseButton)
	self.Guild = wndControl:GetParent():FindChild("Name"):GetText()
	
	local wndSheet = self.wndMain:FindChild("RaidWindow")
	Apollo.LoadForm(self.xmlDoc, self.Boss.."_"..self.Guild , wndSheet, self)
	
	
	for i = 1,3 do
		self.wndMain:FindChild("Guild"..tostring(i)):DestroyChildren()
	end
	
	self:OnUpdateRooster()
	
	if self.save[self.Raid][self.Boss][self.Guild] == nil then
		return
	end
	local savetable = self:Split(self.save[self.Raid][self.Boss][self.Guild], "_")
	if self.Raid == "Datascape" then
		if self.Boss == "Gloomclaw" then
			if self.Guild == "Elysium" then
				local slots = 13
				local wndSheet = self.wndMain:FindChild(self.Boss.."_"..self.Guild)
				for i = 1,slots do
					wndSheet:FindChild("L"..tostring(i)):SetText(savetable[2*i +3])
					wndSheet:FindChild("R"..tostring(i)):SetText(savetable[2*i +4])
				end
			else
			end
		elseif self.Boss == "Lattice" then
			if self.Guild == "Elysium" then
				local slots = 7
				local wndSheet = self.wndMain:FindChild(self.Boss.."_"..self.Guild)
				for i = 1,slots do
					wndSheet:FindChild("N"..tostring(i)):SetText(savetable[3*i +2])
					wndSheet:FindChild("S"..tostring(i)):SetText(savetable[3*i +3])
					wndSheet:FindChild("D"..tostring(i)):SetText(savetable[3*i +4])
				end
				wndSheet:FindChild("D8"):SetText(savetable[26])
			else
			end
		elseif self.Boss == "Limbo" then
			if self.Guild == "Elysium" then
				local slots = 7
				local wndSheet = self.wndMain:FindChild(self.Boss.."_"..self.Guild)
				for i = 1,slots do
					wndSheet:FindChild("G1"..tostring(i)):SetText(savetable[3*i +2])
					wndSheet:FindChild("G2"..tostring(i)):SetText(savetable[3*i +3])
					wndSheet:FindChild("G3"..tostring(i)):SetText(savetable[3*i +4])
				end
			else
			end
		elseif self.Boss == "SD" then
			if self.Guild == "Elysium" then
				local slots = 9
				local wndSheet = self.wndMain:FindChild(self.Boss.."_"..self.Guild)
				--North/South
				for i = 2,slots do
					wndSheet:FindChild("N"..tostring(i-1)):SetText(savetable[2*i+1])
					wndSheet:FindChild("S"..tostring(i-1)):SetText(savetable[2*i+2])
				end
				--Controll Group
				slots = 6
				for i = 2,slots do
					wndSheet:FindChild("C"..tostring(i-1)):SetText(savetable[20+ i])
				end
			else
			end
		elseif self.Boss == "Avatus" then
			if self.Guild == "Elysium" then
				local slots = 11 --the save table for Avatus_Elysium has 40 slots and 4 extra for RS and Guild in the beginning
				local wndSheet = self.wndMain:FindChild(self.Boss.."_"..self.Guild)
				for i = 2,slots do
					wndSheet:FindChild("Y"..tostring(i-1)):SetText(savetable[4*i -3])
					wndSheet:FindChild("G"..tostring(i-1)):SetText(savetable[4*i -2])
					wndSheet:FindChild("R"..tostring(i-1)):SetText(savetable[4*i -1])
					wndSheet:FindChild("B"..tostring(i-1)):SetText(savetable[4*i   ])
				end
			elseif self.Guild == "Bloodpact" then
				local slots = 11 --the save table for Avatus_Elysium has 40 slots and 4 extra for RS and Guild in the beginning
				local wndSheet = self.wndMain:FindChild(self.Boss.."_"..self.Guild)
				for i = 2,slots do
					wndSheet:FindChild("Y"..tostring(i-1)):SetText(savetable[4*i -3])
					wndSheet:FindChild("G"..tostring(i-1)):SetText(savetable[4*i -2])
					wndSheet:FindChild("R"..tostring(i-1)):SetText(savetable[4*i -1])
					wndSheet:FindChild("B"..tostring(i-1)):SetText(savetable[4*i   ])
				end
				wndSheet:FindChild("YB1"):SetText(savetable[45])
				wndSheet:FindChild("YB2"):SetText(savetable[46])
			else
			end
		else
		end
	else
		if self.Boss == "Swabbie" then
			if self.Guild == "Elysium" then
				local slots = 7
				local wndSheet = self.wndMain:FindChild(self.Boss.."_"..self.Guild)
				for i = 1,slots do
					wndSheet:FindChild("G1"..tostring(i)):SetText(savetable[3*i +2])
					wndSheet:FindChild("G2"..tostring(i)):SetText(savetable[3*i +3])
					wndSheet:FindChild("G3"..tostring(i)):SetText(savetable[3*i +4])
				end
			else
			end
		else
		end
	end
end

---------------------------------------------------------------------------------------------------
-- Avatus_Elysium Functions
---------------------------------------------------------------------------------------------------

function Raid_Sheet:OnNewPlayer( wndHandler, wndControl)
	local Parent = wndControl:GetParent()
	local DPS = Parent:FindChild("DPS1"):GetText()
	local Heal = Parent:FindChild("Heal1"):GetText()
	local Tank = Parent:FindChild("Tank1"):GetText()
	if DPS == "" and Heal == "" and Tank == "" then
		return
	end
	
	if self.wndPalyer ~= nil then
		self.wndPalyer:Destroy()
	end
	self.selectedField = wndControl
	self.wndPalyer = Apollo.LoadForm(self.xmlDoc, "PlayerSelect", self.wndMain:FindChild(self.Boss.."_"..self.Guild), self)
	local a, b, c ,d = wndControl:GetAnchorOffsets()
	local PlayerList = self.wndPalyer:FindChild("PlayerList")
	PlayerList:SetBGOpacity(1)
	
	self.wndPalyer:SetAnchorOffsets(a ,b +17 ,a + 130, b +147)
	DPS = self:Split(DPS, ",\n")
	for k,v in pairs(DPS) do
		local  player = Apollo.LoadForm(self.xmlDoc, "Player", PlayerList, self)
		player:FindChild("PlayerName"):SetText(v)
	end
	
	
	Heal = self:Split(Heal, ",\n")
	for k,v in pairs(Heal) do
		local  player = Apollo.LoadForm(self.xmlDoc, "Player", PlayerList, self)
		player:FindChild("PlayerName"):SetText(v)
	end
	
	Tank = self:Split(Tank, ",\n")
	for k,v in pairs(Tank) do
		local  player = Apollo.LoadForm(self.xmlDoc, "Player", PlayerList, self)
		player:FindChild("PlayerName"):SetText(v)
	end
	self.wndPalyer:FindChild("PlayerList"):ArrangeChildrenVert(0)
	self.wndPalyer:Show(true)
end

---------------------------------------------------------------------------------------------------
-- PlayerSelect Functions
---------------------------------------------------------------------------------------------------

function Raid_Sheet:DestroyPlayerList( wndHandler, wndControl )
	if self.wndPalyer ~= nil then 
		self.wndPalyer:Destroy()
	end
end

function Raid_Sheet:OnPlayerClick( wndHandler, wndControl, eMouseButton )
	local Name = wndControl:GetParent():FindChild("PlayerName"):GetText()
	Name = self:Split(Name, " ")
	self.selectedField:SetText(Name[1])
	self:DestroyPlayerList()
end

-----------------------------------------------------------------------------------------------
-- Raid_Sheet Instance
-----------------------------------------------------------------------------------------------
local Raid_SheetInst = Raid_Sheet:new()
Raid_SheetInst:Init()


-----------------------------------------------------------------------------------------------
-- THX to the author of raidlooter addon Johan Lindström (Jabbit-EU, Joxye Nadrax / Wildstar)
-----------------------------------------------------------------------------------------------
-- Compatibility: Lua-5.1
function Raid_Sheet:Split(str, pat)
   local t = {}  -- NOTE: use {n = 0} in Lua-5.0
   local fpat = "(.-)" .. pat
   local last_end = 1
   local s, e, cap = str:find(fpat, 1)
   while s do
      if s ~= 1 or cap ~= "" then
	 table.insert(t,cap)
      end
      last_end = e+1
      s, e, cap = str:find(fpat, last_end)
   end
   if last_end <= #str then
      cap = str:sub(last_end)
      table.insert(t, cap)
   end
   return t
end
------------------------------------------------
--To post in Vince raidframes you need to use }=>{"Group1name", 1,2,...,"Group2",4,5,....,".... etc}